=== modified file 'Makefile'
--- Makefile	2008-10-28 18:00:20 +0000
+++ Makefile	2008-12-28 10:20:47 +0000
@@ -45,7 +45,7 @@
 CFLAGS=$(WARN) $(DEBUG) $(FORTIFY) $(COVERAGE) $(OPTIMIZE) \
 	$(LANGUAGE) $(GNUTLS_CFLAGS) $(AVAHI_CFLAGS) $(GPGME_CFLAGS) \
 	-DVERSION='"$(version)"'
-LDFLAGS=$(COVERAGE) $(LINK_FORTIFY)
+LDFLAGS=$(COVERAGE) $(foreach flag,$(LINK_FORTIFY),-Xlinker $(flag))
 
 # Commands to format a DocBook <refentry> document into a manual page
 DOCBOOKTOMAN=cd $(dir $<); xsltproc --nonet --xinclude \
@@ -76,7 +76,7 @@
 PLUGINS=plugins.d/password-prompt plugins.d/mandos-client \
 	plugins.d/usplash plugins.d/splashy plugins.d/askpass-fifo
 CPROGS=plugin-runner $(PLUGINS)
-PROGS=mandos mandos-keygen $(CPROGS)
+PROGS=mandos mandos-keygen mandos-list $(CPROGS)
 DOCS=mandos.8 plugin-runner.8mandos mandos-keygen.8 \
 	plugins.d/mandos-client.8mandos \
 	plugins.d/password-prompt.8mandos mandos.conf.5 \
@@ -163,6 +163,11 @@
 		--expression='s/^\(VERSION="\)[^"]*"$$/\1$(version)"/' \
 		$@
 
+mandos-list: Makefile
+	$(SED) --in-place \
+		--expression='s/^\(version = "\)[^"]*"$$/\1$(version)"/' \
+		$@
+
 mandos.lsm: Makefile
 	$(SED) --in-place \
 		--expression='s/^\(Version:\).*/\1\t$(version)/' \

=== modified file 'TODO'
--- TODO	2008-12-10 01:26:02 +0000
+++ TODO	2008-12-21 19:19:25 +0000
@@ -6,6 +6,9 @@
    klogctl(6, NULL, 0); klogctl(7, NULL, 0);
 ** TODO [#C] IPv4 support
 
+* plugin-runner
+** TODO [#B] use scandir(3) instead of readdir(3)
+
 * mandos (server)
 ** TODO [#B] Log level				:bugs:
 ** TODO /etc/mandos/clients.d/*.conf
@@ -14,6 +17,8 @@
 ** TODO [#B] Run-time communication with server :bugs:
    Probably using D-Bus
    See also [[*Mandos-tools]]
+** Handle non-existing D-Bus server.
+   Also, possibly a "--no-dbus" option?
 *** Client class
 *** Main server
     + SetLogLevel
@@ -30,13 +35,15 @@
    This will not be strictly necessary when the D-Bus interface is
    implemented.
 
-* Mandos-tools/utilities
-  All of this probably using D-Bus
-** TODO List clients
+* mandos-list
+*** Handle no D-Bus server and/or no Mandos server found better
+*** [#B] --dump option
 ** TODO Disable client
 ** TODO Enable client
 ** TODO Reset timer
 
+* Curses interface
+
 * mandos-keygen
 ** TODO "--secfile" option
    Using the "secfile" option instead of "secret"

=== modified file 'mandos'
--- mandos	2008-12-10 01:26:02 +0000
+++ mandos	2008-12-15 02:33:36 +0000
@@ -170,19 +170,9 @@
 # End of Avahi example code
 
 
-def _datetime_to_dbus_struct(dt, variant_level=0):
-    """Convert a UTC datetime.datetime() to a D-Bus struct.
-    The format is special to this application, since we could not find
-    any other standard way."""
-    return dbus.Struct((dbus.Int16(dt.year),
-                        dbus.Byte(dt.month),
-                        dbus.Byte(dt.day),
-                        dbus.Byte(dt.hour),
-                        dbus.Byte(dt.minute),
-                        dbus.Byte(dt.second),
-                        dbus.UInt32(dt.microsecond)),
-                       signature="nyyyyyu",
-                       variant_level=variant_level)
+def _datetime_to_dbus(dt, variant_level=0):
+    """Convert a UTC datetime.datetime() to a D-Bus type."""
+    return dbus.String(dt.isoformat(), variant_level=variant_level)
 
 
 class Client(dbus.service.Object):
@@ -194,18 +184,18 @@
     secret:     bytestring; sent verbatim (over TLS) to client
     host:       string; available for use by the checker command
     created:    datetime.datetime(); (UTC) object creation
-    last_started: datetime.datetime(); (UTC)
-    started:    bool()
+    last_enabled: datetime.datetime(); (UTC)
+    enabled:    bool()
     last_checked_ok: datetime.datetime(); (UTC) or None
     timeout:    datetime.timedelta(); How long from last_checked_ok
                                       until this client is invalid
     interval:   datetime.timedelta(); How often to start a new checker
-    stop_hook:  If set, called by stop() as stop_hook(self)
+    disable_hook:  If set, called by disable() as disable_hook(self)
     checker:    subprocess.Popen(); a running checker process used
                                     to see if the client lives.
                                     'None' if no process is running.
     checker_initiator_tag: a gobject event source tag, or None
-    stop_initiator_tag:    - '' -
+    disable_initiator_tag:    - '' -
     checker_callback_tag:  - '' -
     checker_command: string; External command which is run to check if
                      client lives.  %() expansions are done at
@@ -249,7 +239,7 @@
     interval = property(lambda self: self._interval, _set_interval)
     del _set_interval
     
-    def __init__(self, name = None, stop_hook=None, config=None):
+    def __init__(self, name = None, disable_hook=None, config=None):
         """Note: the 'checker' key in 'config' sets the
         'checker_command' attribute and *not* the 'checker'
         attribute."""
@@ -280,21 +270,21 @@
                             % self.name)
         self.host = config.get("host", "")
         self.created = datetime.datetime.utcnow()
-        self.started = False
-        self.last_started = None
+        self.enabled = False
+        self.last_enabled = None
         self.last_checked_ok = None
         self.timeout = string_to_delta(config["timeout"])
         self.interval = string_to_delta(config["interval"])
-        self.stop_hook = stop_hook
+        self.disable_hook = disable_hook
         self.checker = None
         self.checker_initiator_tag = None
-        self.stop_initiator_tag = None
+        self.disable_initiator_tag = None
         self.checker_callback_tag = None
         self.checker_command = config["checker"]
     
-    def start(self):
+    def enable(self):
         """Start this client's checker and timeout hooks"""
-        self.last_started = datetime.datetime.utcnow()
+        self.last_enabled = datetime.datetime.utcnow()
         # Schedule a new checker to be started an 'interval' from now,
         # and every interval from then on.
         self.checker_initiator_tag = (gobject.timeout_add
@@ -302,42 +292,42 @@
                                        self.start_checker))
         # Also start a new checker *right now*.
         self.start_checker()
-        # Schedule a stop() when 'timeout' has passed
-        self.stop_initiator_tag = (gobject.timeout_add
+        # Schedule a disable() when 'timeout' has passed
+        self.disable_initiator_tag = (gobject.timeout_add
                                    (self._timeout_milliseconds,
-                                    self.stop))
-        self.started = True
+                                    self.disable))
+        self.enabled = True
         # Emit D-Bus signal
-        self.PropertyChanged(dbus.String(u"started"),
+        self.PropertyChanged(dbus.String(u"enabled"),
                              dbus.Boolean(True, variant_level=1))
-        self.PropertyChanged(dbus.String(u"last_started"),
-                             (_datetime_to_dbus_struct
-                              (self.last_started, variant_level=1)))
+        self.PropertyChanged(dbus.String(u"last_enabled"),
+                             (_datetime_to_dbus(self.last_enabled,
+                                                variant_level=1)))
     
-    def stop(self):
-        """Stop this client."""
-        if not getattr(self, "started", False):
+    def disable(self):
+        """Disable this client."""
+        if not getattr(self, "enabled", False):
             return False
-        logger.info(u"Stopping client %s", self.name)
-        if getattr(self, "stop_initiator_tag", False):
-            gobject.source_remove(self.stop_initiator_tag)
-            self.stop_initiator_tag = None
+        logger.info(u"Disabling client %s", self.name)
+        if getattr(self, "disable_initiator_tag", False):
+            gobject.source_remove(self.disable_initiator_tag)
+            self.disable_initiator_tag = None
         if getattr(self, "checker_initiator_tag", False):
             gobject.source_remove(self.checker_initiator_tag)
             self.checker_initiator_tag = None
         self.stop_checker()
-        if self.stop_hook:
-            self.stop_hook(self)
-        self.started = False
+        if self.disable_hook:
+            self.disable_hook(self)
+        self.enabled = False
         # Emit D-Bus signal
-        self.PropertyChanged(dbus.String(u"started"),
+        self.PropertyChanged(dbus.String(u"enabled"),
                              dbus.Boolean(False, variant_level=1))
         # Do not run this again if called by a gobject.timeout_add
         return False
     
     def __del__(self):
-        self.stop_hook = None
-        self.stop()
+        self.disable_hook = None
+        self.disable()
     
     def checker_callback(self, pid, condition, command):
         """The checker has completed, so take appropriate actions."""
@@ -376,14 +366,13 @@
         alive and well.
         """
         self.last_checked_ok = datetime.datetime.utcnow()
-        gobject.source_remove(self.stop_initiator_tag)
-        self.stop_initiator_tag = (gobject.timeout_add
-                                   (self._timeout_milliseconds,
-                                    self.stop))
+        gobject.source_remove(self.disable_initiator_tag)
+        self.disable_initiator_tag = (gobject.timeout_add
+                                      (self._timeout_milliseconds,
+                                       self.disable))
         self.PropertyChanged(dbus.String(u"last_checked_ok"),
-                             (_datetime_to_dbus_struct
-                              (self.last_checked_ok,
-                               variant_level=1)))
+                             (_datetime_to_dbus(self.last_checked_ok,
+                                                variant_level=1)))
     
     def start_checker(self):
         """Start a new checker subprocess if one is not running.
@@ -458,7 +447,7 @@
     
     def still_valid(self):
         """Has the timeout not yet passed for this client?"""
-        if not getattr(self, "started", False):
+        if not getattr(self, "enabled", False):
             return False
         now = datetime.datetime.utcnow()
         if self.last_checked_ok is None:
@@ -497,18 +486,17 @@
                 dbus.String("host"):
                     dbus.String(self.host, variant_level=1),
                 dbus.String("created"):
-                    _datetime_to_dbus_struct(self.created,
-                                             variant_level=1),
-                dbus.String("last_started"):
-                    (_datetime_to_dbus_struct(self.last_started,
-                                              variant_level=1)
-                     if self.last_started is not None
+                    _datetime_to_dbus(self.created, variant_level=1),
+                dbus.String("last_enabled"):
+                    (_datetime_to_dbus(self.last_enabled,
+                                       variant_level=1)
+                     if self.last_enabled is not None
                      else dbus.Boolean(False, variant_level=1)),
-                dbus.String("started"):
-                    dbus.Boolean(self.started, variant_level=1),
+                dbus.String("enabled"):
+                    dbus.Boolean(self.enabled, variant_level=1),
                 dbus.String("last_checked_ok"):
-                    (_datetime_to_dbus_struct(self.last_checked_ok,
-                                              variant_level=1)
+                    (_datetime_to_dbus(self.last_checked_ok,
+                                       variant_level=1)
                      if self.last_checked_ok is not None
                      else dbus.Boolean (False, variant_level=1)),
                 dbus.String("timeout"):
@@ -565,9 +553,9 @@
     def SetTimeout(self, milliseconds):
         self.timeout = datetime.timedelta(0, 0, 0, milliseconds)
     
-    # Start - method
-    Start = dbus.service.method(_interface)(start)
-    Start.__name__ = "Start"
+    # Enable - method
+    Enable = dbus.service.method(_interface)(enable)
+    Enable.__name__ = "Enable"
     
     # StartChecker - method
     @dbus.service.method(_interface)
@@ -575,11 +563,11 @@
         "D-Bus method"
         self.start_checker()
     
-    # Stop - method
+    # Disable - method
     @dbus.service.method(_interface)
-    def Stop(self):
+    def Disable(self):
         "D-Bus method"
-        self.stop()
+        self.disable()
     
     # StopChecker - method
     StopChecker = dbus.service.method(_interface)(stop_checker)
@@ -1077,8 +1065,8 @@
         
         while clients:
             client = clients.pop()
-            client.stop_hook = None
-            client.stop()
+            client.disable_hook = None
+            client.disable()
     
     atexit.register(cleanup)
     
@@ -1119,7 +1107,7 @@
         def RemoveClient(self, object_path):
             for c in clients:
                 if c.dbus_object_path == object_path:
-                    c.stop()
+                    c.disable()
                     clients.remove(c)
                     return
             raise KeyError
@@ -1132,7 +1120,7 @@
         # Emit D-Bus signal
         mandos_server.ClientAdded(client.dbus_object_path,
                                   client.GetAllProperties())
-        client.start()
+        client.enable()
     
     tcp_server.enable()
     tcp_server.server_activate()

=== added file 'mandos-list'
--- mandos-list	1970-01-01 00:00:00 +0000
+++ mandos-list	2008-12-21 19:19:25 +0000
@@ -0,0 +1,65 @@
+#!/usr/bin/python
+# -*- mode: python; coding: utf-8 -*-
+
+import dbus
+from optparse import OptionParser
+import locale
+
+locale.setlocale(locale.LC_ALL, u'')
+
+tablewords = {
+    'name': u'Name',
+    'enabled': u'Enabled',
+    'timeout': u'Timeout',
+    'last_checked_ok': u'Last Successful Check',
+    'created': u'Created',
+    'interval': u'Interval',
+    'host': u'Host',
+    'fingerprint': u'Fingerprint',
+    'checker_running': u'Check Is Running',
+    'last_enabled': u'Last Enabled',
+    'checker': u'Checker',
+    }
+busname = 'org.mandos-system.Mandos'
+object_path = '/Mandos'
+interface = 'org.mandos_system.Mandos'
+version = "1.0.2"
+defaultkeywords = ('name', 'enabled', 'timeout', 'last_checked_ok',
+                   'checker')
+
+parser = OptionParser(version = "%%prog %s" % version)
+parser.add_option("-a", "--all", action="store_true", default=False,
+                  help="Print all fields")
+options = parser.parse_args()[0]
+if options.all:
+    keywords = ('name', 'enabled', 'timeout', 'last_checked_ok',
+                'created', 'interval', 'host', 'fingerprint',
+                'checker_running', 'last_enabled', 'checker')
+else:
+    keywords = defaultkeywords
+
+
+bus = dbus.SystemBus()
+mandos_dbus_objc = bus.get_object(busname, object_path)
+mandos_serv = dbus.Interface(mandos_dbus_objc,
+                             dbus_interface = interface)
+mandos_clients = mandos_serv.GetAllClientsWithProperties()
+
+def valuetostring(x):
+    if type(x) is dbus.Boolean:
+        return u"Yes" if x else u"No"
+    else:
+        return unicode(x)
+
+format_string = u' '.join(u'%%-%ds'
+                         % max(len(tablewords[key]),
+                               max(len(valuetostring(client[key]))
+                                   for client
+                                   in mandos_clients.itervalues()))
+                         for key in keywords)
+print format_string % tuple(tablewords[key] for key in keywords) 
+for client in mandos_clients.itervalues():
+    print format_string % tuple(valuetostring(client[key])
+                                for key in keywords)
+
+

